QUERY = [
    "hold on, fetching data from the Matrix...",
    "let me consult the command oracle...",
    "gathering ancient shell wisdom...",
    "contacting the shell gurus on the other side...",
    "talking to the shell spirits, one moment please...",
    "scouring the shell dungeons for answers...",
    "scanning the shell multiverse...",
    "whispering to the shell command fairies...",
    "asking the shell command crystal ball...",
    "brewing a potion to summon the right command...",
    "reading the command tarot cards...",
    "channeling the command force, stay tuned...",
]

EXPLAIN = [
    "let me decrypt this shell spell for you...",
    "translating command hieroglyphs...",
    "hold on, I'm unlocking the shell command secrets...",
    "it's shell command storytime, one moment...",
    "digging up the hidden meaning of this command...",
    "shell command mysteries, being revealed shortly...",
    "peeling back the layers of this shell command onion...",
    "decoding the command enigma...",
    "cracking open the command vault of knowledge...",
    "navigating the shell command labyrinth...",
    "solving the command riddle for you...",
    "assembling the command puzzle pieces...",
]
